using System;

namespace _3DEngine
{

	public class Matrix
	{

		static Matrix() 
		{
			addSubCount = 0;
			mulDivCount = 0;
			trigCount = 0;
			isLogging = false;
		}

		private static long addSubCount;
		private static long mulDivCount;
		private static long trigCount;
		private static bool isLogging;

		public static void BeginOperationLog() 
		{
			addSubCount = 0;
			mulDivCount = 0;
			trigCount = 0;
			isLogging = true;
		}

		public static void EndOperationLog() 
		{
			isLogging = false;
		}

		public static long GetAddSubCount() 
		{
			return addSubCount;
		}

		public static long GetMulDivCount() 
		{
			return mulDivCount;
		}

		public static long GetTrigCount() 
		{
			return trigCount;
		}

		internal double[,] x;

		internal Matrix(double[,] x)
		{
			this.x = x;
		}

		public Matrix Mult(Matrix matrix) 
		{
			double[,] result = new double[4,4];
			for(int i = 0; i < 4; i++) 
			{
				for(int j = 0; j < 4; j++) 
				{
					double sum = 0;
					for(int k = 0; k < 4; k++) 
					{
						sum += x[i,k] * matrix.x[k,j];
						if(isLogging) 
						{
							addSubCount++;
							mulDivCount++;
						}
					}
					result[i,j] = sum;
				}
			}
			return new Matrix(result);
		}

		public Vector Mult(Vector vector)
		{
			double[] result = new double[4];
			for(int i = 0; i < 4; i++)
			{
				double sum = 0;
				for(int j = 0; j < 4; j++) 
				{
					sum += x[i,j] * vector.x[j];
					if(isLogging) 
					{
						addSubCount++;
						mulDivCount++;
					}
				}
				result[i] = sum;
			}
			return new Vector(result);
		}

		public static Matrix CreateIdentityMatrix() 
		{
			double[,] result = {{1,0,0,0},{0,1,0,0},{0,0,1,0},{0,0,0,1}};
			return new Matrix(result);
		}

		public static Matrix CreateTranslationMatrix(double x, double y, double z) 
		{
			double[,] result = {{1,0,0,x},{0,1,0,y},{0,0,1,z},{0,0,0,1}};
			return new Matrix(result);
		}

		public static Matrix CreateScalingMatrix(double s) 
		{
			double[,] result = {{s,0,0,0},{0,s,0,0},{0,0,s,0},{0,0,0,1}};
			return new Matrix(result);
		}

		public static Matrix CreateRotateXMatrix(double a) 
		{
			if(isLogging) 
			{
				trigCount +=2;
			}
			double sin = Math.Sin(a);
			double cos = Math.Cos(a);
			double[,] result = {{1,0,0,0},{0,cos,sin,0},{0,-sin,cos,0},{0,0,0,1}};
			return new Matrix(result);
		}

		public static Matrix CreateRotateYMatrix(double a) 
		{
			if(isLogging) 
			{
				trigCount +=2;
			}
			double sin = Math.Sin(a);
			double cos = Math.Cos(a);
			double[,] result = {{cos,0,-sin,0},{0,1,0,0},{sin,0,cos,0},{0,0,0,1}};
			return new Matrix(result);
		}

		public static Matrix CreateRotateZMatrix(double a) 
		{
			if(isLogging) 
			{
				trigCount +=2;
			}
			double sin = Math.Sin(a);
			double cos = Math.Cos(a);
			double[,] result = {{cos,sin,0,0},{-sin,cos,0,0},{0,0,1,0},{0,0,0,1}};
			return new Matrix(result);
		}

	}

}
