using System;

namespace _3DEngine
{

	public class Point : ITransformable
	{

		static Point() 
		{
			addSubCount = 0;
			mulDivCount = 0;
			sqrtCount = 0;
			isLogging = false;
		}

		private static long addSubCount;
		private static long mulDivCount;
		private static long sqrtCount;
		private static bool isLogging;

		public static void BeginOperationLog() 
		{
			addSubCount = 0;
			mulDivCount = 0;
			sqrtCount = 0;
			isLogging = true;
		}

		public static void EndOperationLog() 
		{
			isLogging = false;
		}

		public static long GetAddSubCount() 
		{
			return addSubCount;
		}

		public static long GetMulDivCount() 
		{
			return mulDivCount;
		}

		public static long GetSqrtCount() 
		{
			return sqrtCount;
		}

		private Vector vector;
		private double x;
		private double y;
		private double z;

		public Point(double x, double y, double z)
		{
			vector = new Vector(x, y, z);
			this.x = x;
			this.y = y;
			this.z = z;
		}

		public void Transform(Matrix transformation) 
		{
			Vector v = transformation.Mult(vector);
			Point point = v.GetPoint();
			x = point.GetX();
			y = point.GetY();
			z = point.GetZ();
		}

		public double GetX()
		{
			return x;
		}

		public double GetY()
		{
			return y;
		}

		public double GetZ()
		{
			return z;
		}

		public override string ToString() 
		{
			return "(" + x + ":" + y + ":" + z + ")";
		}

		public double Mult(Point p) 
		{
			if(isLogging) 
			{
				addSubCount += 3;
				mulDivCount += 3;
			}
			return x*p.x + y*p.y + z*p.z;
		}

		public Point Sub(Point p) 
		{
			if(isLogging) 
			{
				addSubCount += 3;
				mulDivCount += 3;
			}
			return new Point(x-p.x, y-p.y, z-p.z);
		}

		public double GetLength() 
		{
			if(isLogging) 
			{
				sqrtCount ++;
				addSubCount += 3;
				mulDivCount += 3;
			}
			return Math.Sqrt(x*x + y*y + z*z);
		}

	}

}
